/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import datatypes.List;
import datastructures.Node;

public class DoublyLinkedList implements List {
	private Node node;
	private int index;
 	private int size;

	// The constructor. Takes no arguments. Initilizes the member variables
	public DoublyLinkedList() {
		this.node = null;
		this.index = -1;
		this.size = 0;
	}

	// sets the current node to the node at position 'index'
	private void moveToNode(int index) {
		// if the current position is before the index, move forward
		// until we are at the same position
		while (this.index < index) {
			this.index++;

			if (this.index < size) {
				this.node = this.node.getNextNode();
			}
		}

		// if the current position is after the index, move backward
		// until we are at the same position
		while (this.index > index) {
			this.index--;

			if (this.node.getLastNode() != null) {
				this.node = this.node.getLastNode();
			}
		}
	}

	// adds a new Node with value 'element' at position index
	public void insert(int index, Object element) {
		Node newNode;
		int size = this.size();

		// if index is beyond the end of the list, change it to the end
		if (index > size) {
			index = size;
		}

		// if index is before the beginning of the list, change
		// it to the beginning
		if (index < 0) {
			index = 0;
		}

		if (size == 0) {
			// if there are no elements in the list. make a new
			// beginning last and next are null pointers

			this.node = new Node(element, null, null);
			this.index++;
		} else {
			this.moveToNode(index);

			// now insert the node based on the index

			if (this.index == size) {
				// if we are inserting at the end of the list
				// newNode has a last pointer to the node, and
				// a null next pointer

				newNode = new Node(element, this.node, null);
				this.node.setNextNode(newNode);
				this.node = this.node.getNextNode();
			} else if (this.index == 0) {
				// if we are inserting at the beginning of the list
				// newNode has a null last pointer, and a next
				// pointer to the current node

				newNode = new Node(element, null, this.node);
				this.node.setLastNode(newNode);
				this.node = this.node.getLastNode();
			} else {
				// if we are inserting into the middle of the list
				// newNode has a last pointer to the previous node
				// and a next pointer to the current node

				newNode = new Node(element, this.node.getLastNode(), this.node);
				this.node.getLastNode().setNextNode(newNode);
				this.node.setLastNode(newNode);
				this.node = this.node.getLastNode();
			}
		}

		this.size++;
	}

	// get returns the value of the Node at position index
	public Object get(int index) {
		int size = this.size();

		if (index >= size || index < 0) {
			// invalid index
			return null;
		}

		this.moveToNode(index);

		return this.node.getElement();
	}

	// deletes the node at position 'index'.
	public Object delete(int index) {
		Object element;
		int size = this.size();

		if (index >= size || index < 0) {
			// invalid index
			return null;
		}

		this.moveToNode(index);
		element = this.node.getElement();

		// now delete the node based on its position in the list

		if (this.index == 0) {
			// if we are deleting the first node

			if (this.size() == 1) {
				// if there is only one Node in the list,
				// then delete the reference node

				this.node = null;
			} else {
				// otherwise, tell the next node to set its last
				// node null

				this.node.getNextNode().setLastNode(null);
				this.node = this.node.getNextNode();
			}
		} else if (this.index == (size - 1)) {
			// if we are deleting the last Node (size - 1)
			// tell the last node to set its next node null

			this.node.getLastNode().setNextNode(null);
			this.node = this.node.getLastNode();
			this.index--;
		} else {
			// if we are deleting from the middle of the list
			// the last node sets it next node pointer to be the nodes
			// next node, and the next nodes last node is set to be
			// this nodes last node

			this.node.getLastNode().setNextNode(this.node.getNextNode());
			this.node.getNextNode().setLastNode(this.node.getLastNode());
			this.node = this.node.getNextNode();
		}

		this.size--;

		return element;
	}

	// return the index of the element in the list
	public int indexOf(Object element) {
		this.moveToNode(0);

		// loop through the elements from beginning to end until we
		// find the element.

		while (this.index < this.size()) {
			if (this.node.getElement().equals(element)) {
				// if we found it, return the index

				return this.index;
			} else {
				// otherwise, try the next node

				this.index++;

				if (this.node.getNextNode() != null) {
					this.node = this.node.getNextNode();
				}
			}
		}

		// if we reached this point, we didn't find it
		return -1;
	}

	// returns the list size
	public int size() {
		return this.size;
	}
}

