/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

public class KeyTree {
	private Comparable key;
	private KeyTree leftChild, rightChild, parent;
	private Object value;

	// the constructor for the root (null parent = root)
	public KeyTree(Comparable key, Object value) {
		this(key, value, null);
	}

	// the constructor (no tree will ever know what children it will have)
	public KeyTree(Comparable key, Object value, KeyTree parent) {
		this.key = key;
 		this.leftChild = null;
		this.rightChild = null;
		this.value = value;
		this.parent = parent;
 	}

	// tests to see whether or not this tree is the root
	public boolean isRoot() {
		return (this.getParent() == null);
	}

	// tests to see whether this node is the left child of its parent
	// if its not, then it must be the right child
	public boolean isLeftChild() {
		Comparable key, leftChildKey;

		if (this.isRoot()) {
			// if we are root, then we are not a child

			return false;
		} else if (this.getParent().getLeftChild() == null) {
			// if our parent has no left child, then we cannot
			// be the left child

			return false;
		}

		key = this.getKey();
		leftChildKey = this.getParent().getLeftChild().getKey();

		if (key.compareTo(leftChildKey) != 0) {
			// if my key doesn't match my parent's left child,
			// then I can't be the left child

			return false;
		}

		// if we didn't eliminate me, then I must be it
		return true;
	}

	// tests to see whether this tree has a left child
	public boolean hasLeftChild() {
		return (this.getLeftChild() != null);
	}

	// tests to see whether this tree has a right child
	public boolean hasRightChild() {
		return (this.getRightChild() != null);
	}

	// tests to see whether this tree has no children
	public boolean hasNoChildren() {
		return (this.getLeftChild() == null && this.getRightChild() == null);
	}

	// tests to see whether this tree only has a left child
	public boolean onlyLeftChild() {
		return (this.getLeftChild() != null && this.getRightChild() == null);
 	}

	// tests to see whether this tree only has a right child
	public boolean onlyRightChild() {
		return (this.getLeftChild() == null && this.getRightChild() != null);
	}

	// tests to see if this tree has only one child (left or right)
	public boolean onlyOneChild() {
		return (this.onlyRightChild() || this.onlyLeftChild());
	}

	// returns the Key for this tree
	public Comparable getKey() {
		return this.key;
	}

	// sets the key for this tree
	public void setKey(Comparable key) {
		this.key = key;
	}

	// returns the left child of this tree
	public KeyTree getLeftChild() {
		return this.leftChild;
	}

	// sets the left child of this tree
	public void setLeftChild(KeyTree leftChild) {
		this.leftChild = leftChild;
	}

	// returns the right child of this tree
	public KeyTree getRightChild() {
		return this.rightChild;
	}

	// sets the right child of this tree
	public void setRightChild(KeyTree rightChild) {
		this.rightChild = rightChild;
	}

	// returns the parent of this tree
	public KeyTree getParent() {
		return this.parent;
	}

	// sets the parent of this tree
	public void setParent(KeyTree parent) {
		this.parent = parent;
	}

	// returns the value of this tree
	public Object getValue() {
		return this.value;
	}

	// sets the value of this tree
	public void setValue(Object value) {
		this.value = value;
	}
}

