/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package model;

import datatypes.List;
import datatypes.Dictionary;

//////////////////////////////////////////////////////////////////////
/**
 * The LineManager class represents and manages a list of lines.
 * A current line can be set for this list.  Many of the 
 * operations supported, only operate on the current line.  Lines
 * can be inserted and deleted.  Individual words can be inserted
 * into and deleted from lines.  These words can also be searched for
 * and replaced.
 **/
//////////////////////////////////////////////////////////////////////
public class LineManager
    {
    private LineList lineList;
    private WordDictionary wordDictionary;
    private int currentLine;

    //////////////////////////////////////////////////////////////////////
    /**
     * Initialize this LineManager.  The List is passed in so that we can 
     * swap out this DoublyLinkedList that students write with something 
     * else and it should still work (as stated in the assignment).
     *
     * @param lineList the list that will hold all lines
     * @param dictionary the dictionary that will hold all words
     */
    public LineManager(List lineList, Dictionary dictionary)
        {
	this.lineList = new LineList(lineList);
	wordDictionary = new WordDictionary(dictionary);
	currentLine = 0;
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the current line number.
     *
     * @return the current line number
     */
    public int getCurrentLineNumber()
        {
	return currentLine;
        }
    //////////////////////////////////////////////////////////////////////     
    /** 
     * Set the current line number.  Throws an IndexOutOfBoundsException
     * if the newLine is less than 0 or greater than the the current
     * number of lines.
     *
     * @param newLine the new curren line number.
     */
    public void setCurrentLineNumber(int newLine)
        throws IndexOutOfBoundsException
        {
        if ((newLine > getTotalNumberOfLines()) || (newLine < 0)) 
            {
            String message = "Invalid index:"+newLine+".";
            throw new IndexOutOfBoundsException(message);
            }
        else
            {
            this.currentLine = newLine;
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the total number of lines in this block of text.
     *
     * @return the total number of lines
     **/
    public int getTotalNumberOfLines()
        {
	return lineList.size();
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Insert the line at the current line number.  We have to create a new 
     * Line object and insert it into the list.  We then have to add each word
     * of the BinarySearchTree.
     *
     * @param text a line of text to insert
     * @param lineNumber the index to insert the line of text before.
     **/
    public void insertLine(String text, int lineNumber)
        throws IndexOutOfBoundsException 
        {
	Line line = new Line(text);
	lineList.insert(lineNumber, line);
        
	Word currentWord;
        //Now we have to add each word in the line to the tree.
	for(int i = 0; i < line.size(); i++)
            {
	    currentWord = line.getWordAt(i);
	    wordDictionary.insertWord(currentWord);
            }
        setCurrentLineNumber(lineNumber + 1);
    }

    //////////////////////////////////////////////////////////////////////
    /**
     * Insert the given word at the specified index on the specified line.
     *
     * @param text a word to insert
     * @param lineNumber the index of the line to inser the word into.
     * @param index the index into a line at whic a word should be inserted.
     */
    public void insertWordInLine(int index, String text, int lineNumber) 
        throws IndexOutOfBoundsException
        {
        Line currentLine = lineList.getLineAt(lineNumber);
        Word newWord = new Word(text, currentLine);
        currentLine.insertWord(newWord, index);
        wordDictionary.insertWord(newWord);
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Delete the first occurrence of the specified word from the 
     * specified line.  If the word does not exist, do nothing.
     *
     * @param text a word to delete from the specified line.
     * @param lineNumber the line number to delete the line from.
     **/
    public void deleteWordFromLine(String text, int lineNumber)
        {
        Line line = lineList.getLineAt(lineNumber);
        Word[] words = wordDictionary.findAllWords(text);

        //the largest index possible index 
        int firstIndex = Integer.MAX_VALUE;

        //set nullIndex = firstIndex...if no words matching
        //text are on this list, firstIndex and nullIndex
        //will still be the same
        int nullIndex = firstIndex;
        for (int i = 0; i < words.length; i++)
            {
            Word word = words[i];
            //make sure word is on the proper line
            if (word.getLine().equals(line))
                {
                //make sure index is the smallest one
                int wordIndex = line.indexOf(word);
                if (wordIndex < firstIndex)
                    {
                    firstIndex = wordIndex;
                    }
                }
            }
        //if word was found on this line...
        if (firstIndex != nullIndex)
            {
            Word firstWord = line.getWordAt(firstIndex);
            firstWord.getLine().deleteWord(firstWord);
            wordDictionary.deleteWord(firstWord);
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Delete the text lines between <em>from</em> and <em>to</em> inclusive.
     * Set the current line to the line after the last deleted line.
     * We must also delete the words of each line from the tree.
     * The line numbers must be valid, that is, they must be > 0 and
     * < getTotalNumberOfLines(), and from < to.  If this is not the case
     * an IndexOutOfBoundsException will be thrown.
     *
     * @param from the first line to delete.
     * @param to the last line to delete
     **/
    public void deleteLines(int from, int to)
        throws IndexOutOfBoundsException
        {
        //if from is larger than to
        if (from > to || from < 0 || to >= getTotalNumberOfLines()) 
            {
            throw new IndexOutOfBoundsException();
            }
        else
            {
            Line currentLine;
            for(int i = from; i <= to; i++)
                {
                currentLine = lineList.getLineAt(from);
                //always delete from...since we are deleting 
                //the index decreases by one
                lineList.delete(currentLine);
                for(int j = 0; j < currentLine.size(); j++)
                    {
                    wordDictionary.deleteWord(currentLine.getWordAt(j));
                    }
                }	
            //Set the current line number to the end of deletion.
            setCurrentLineNumber(from);
            }
        }
    //////////////////////////////////////////////////////////////////////
    /**
     * Replace the first occurrence of a given word with another word.
     * Note: this method does not change the current line.
     *
     * @param replacee the word to replace
     * @param replacer the word that replacee will be replaced with.
     */
    public void replaceWord(String replacee, String replacer)
        {
        //grab the current line
        int currentLine = getCurrentLineNumber();
        WordPosition wordPosition = find(replacee);

        //reset the current line, since find sets it.
        setCurrentLineNumber(currentLine);
        //make sure the replacee is present
        if (wordPosition != null)
            {
            int lineNumber = wordPosition.getLineNumber();
            int index = wordPosition.getIndex();
            deleteWordFromLine(replacee, lineNumber);
            insertWordInLine(index, replacer, lineNumber);
            }
        }
    //////////////////////////////////////////////////////////////////////
    /**
     * Returns the line number and index in the line of the first 
     * occurrance of the given word in the text.  Also, set the current
     * line to the line number of the found word.  If no word is found
     * do not change the line, and return null.
     *
     * @param text find the specified string within the list of lines.
     * @return the current position (lineNumber, index) of text
     */
    public WordPosition find(String text)
        {
        Word[] currentWords = wordDictionary.findAllWords(text);
        //if no word found, return null.
        if (currentWords.length == 0)
            {
            return null;
            }
        //create a word position object to represent the largest possible
        //position that a word could be placed.
        Word firstWord = findFirstOccurenceOfWord(currentWords);
        //get the word's index
        int wordIndex = firstWord.getLine().indexOf(firstWord);
        //get the line number
        int lineNumber = lineList.indexOf(firstWord.getLine());
        WordPosition position = new WordPosition(wordIndex, lineNumber);
        //set the current line equal to where the word was found...
        setCurrentLineNumber(position.getLineNumber());
        return position;
        }
    //////////////////////////////////////////////////////////////////////
    /**
     * Return a strings representing the current line.
     * @return the line at index i.
     */
    public String getLineAt(int i)
        {
        return lineList.getLineAt(i).toString();
        }
    //////////////////////////////////////////////////////////////////////
    /**
     * Returns the first occurrence of the value that this word list
     * holds.  First occurrence is defined to the be first word whose
     * value matches the word lists, when searching from the first word
     * on the first line.
     */
    private Word findFirstOccurenceOfWord(Word[] words)
        {
        //create a word position object to represent the largest possible
        //position that a word could be placed.
        WordPosition firstOccurence = new WordPosition(Integer.MAX_VALUE,
                                                      getTotalNumberOfLines());
        Word firstWord = null;
        for (int i = 0; i < words.length; i++)
            {
            Word currentWord = words[i];
            int lineNumber = lineList.indexOf(currentWord.getLine());
            int index = currentWord.getLine().indexOf(currentWord);
            //if this word's parent line index is less than our previous
            //smallest line...then set this to be our new smallest line
            if (lineNumber < firstOccurence.getLineNumber())
                { 
                firstWord = currentWord;
                firstOccurence = new WordPosition(index, lineNumber);
                }
            //if this word's line is the same as our previous earlist word
            //and its word index w/i that line is less...then set this
            //to be our new smallest line
            else if ((lineNumber == firstOccurence.getLineNumber()) &&
                     (index < firstOccurence.getIndex()))
                {
                firstOccurence = new WordPosition(index, lineNumber);
                firstWord = currentWord;
                }
            }
        return firstWord;
        }
    }


