/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package model;

// EXTRA CREDIT: WE IMPLEMENTED THE $LAST_LINE AS A PARAMETER IN THE replaceLastLine()
//		 method, tryToDisplay(), tryToDelete(), and tryToSetCurrentLine() methods

import java.util.StringTokenizer;

import datatypes.List;
import datatypes.Dictionary;

import driver.AutomationObject;

import model.MVC_Model;
import model.LineManager;

import view.MVC_View;
import view.VisualEditor;

public class Model implements MVC_Model {
	private MVC_View view;
	private LineManager document;

	private boolean inInsertMode;

	// Editor commands
	private final static char COMMAND_START = '$';
	private final static String INSERT_COMMAND = "INSERT_LINES";
	private final static String DELETE_COMMAND = "DELETE_LINES";
	private final static String SETLINE_COMMAND = "SET_LINE";
	private final static String DONE_COMMAND = "DONE";
	private final static String DISPLAY_COMMAND = "DISPLAY_LINES";
	private final static String INSERT_WORD_COMMAND = "INSERT_WORD";
	private final static String DELETE_WORD_COMMAND = "DELETE_WORD";
	private final static String FIND_COMMAND = "FIND_WORD";
	private final static String REPLACE_COMMAND = "REPLACE_WORD";	
	private final static String LAST_LINE_COMMAND = "LAST_LINE";

	// Editor errors
	private final static String BAD_COMMAND = "BAD COMMAND";
        private final static String FIND_ERROR = "GIVEN STRING NOT FOUND";
	private final static String MALFORMED_RANGE = "MALFORMED RANGE";
	private final static String OUT_OF_RANGE = "OUT OF RANGE";
        private final static String NO_LINES_PRESENT = "NO LINES PRESENT";
	private final static String INTEGER_EXPECTED = "INTEGER EXPECTED";
	private final static String NO_ARGUMENTS_EXPECTED = "NO ARGUMENTS EXPECTED";
	private final static String ONE_INTEGER_EXPECTED = "ONE INTEGER EXPECTED";
	private final static String TWO_INTEGERS_EXPECTED = "TWO INTEGERS EXPECTED";
	private final static String INTEGER_FORMAT_ERROR = "INTEGER FORMAT ERROR";
	private final static String TWO_ARGUMENTS_EXPECTED = "TWO ARGUMENTS EXPECTED";
	private final static String ONE_ARGUMENT_EXPECTED = "ONE ARGUMENT EXPECTED";

	public Model(List list, Dictionary dictionary, AutomationObject automation) {
		this.document = new LineManager(list, dictionary);
		this.view = new VisualEditor(automation, this);

		this.inInsertMode = false;
	}

	// process strings from the Controller
	public void updateData(String command) {
		if ((command.length() == 0) && (!this.inInsertMode)) {
			this.displayError(Model.BAD_COMMAND);
		} else if ((command.length() == 0) && (this.inInsertMode)) {
			this.insert(command);
		} else if (command.charAt(0) == Model.COMMAND_START) {
			this.commandCheck(command);
		} else if (this.inInsertMode) {
			this.insert(command);
		} else {
			this.displayError(Model.BAD_COMMAND);
		}
	}

	// adds HTML tags to the string and highlights the current line
	// <br> tags are used to separate each line
	private String prepareBuffer() {
		return "<html><body>\n" + this.prepareBuffer(0, this.document.getTotalNumberOfLines()) + "\n</body></html>";
	}

	// highlights the current line blue, but does not add document tags so we can assemble
	// partial bits of the document and sew them together later (see display(from, to))
	private String prepareBuffer(int from, int to) {
		String line, string = "";
		int loop;

		for (loop = from; loop < to; loop++) {
			line = document.getLineAt(loop);

			if (loop == document.getCurrentLineNumber()) {
				// current line is highlighted blue
				string = string + "<font color=blue><strong>\n" + line + "\n</strong></font><br>\n";
			} else {
				string = string + line + "\n<br>\n";
			}
		}

		return string;
	}

	// errors are so prevalent, they deserve their own method ;->
	private void displayError(String errorMsg) {
		this.view.updateView(errorMsg, true);
	}

	// replace the $LAST_LINE command within the tokens with the last line in the document
	private StringTokenizer replaceLastLine(StringTokenizer tokens) {
		String token, newString = "";
		boolean atStart = true;

		while (tokens.hasMoreTokens()) {
			token = tokens.nextToken();

			if (token.equals(Model.COMMAND_START + Model.LAST_LINE_COMMAND)) {
				if (!atStart) {
					newString = newString + " " + (document.getTotalNumberOfLines()-1);
				} else {
					newString = newString + (document.getTotalNumberOfLines()-1);
					atStart = false;
				}
			} else {
				if (!atStart) {
					newString = newString + " " + token;
				} else {
					newString = token;
					atStart = false;
				}
			}
		}

		if (newString.length() > 0) {
			tokens = new StringTokenizer(newString);
		}

		return tokens;
	}

	// check to see if we received a command. If so, process it.
	// We also have the $LAST_LINE extra credit parameter implemented here.
	private void commandCheck(String string) {
		StringTokenizer tokens = new StringTokenizer(string);
		String command = tokens.nextToken();

		command = command.substring(1);
		this.inInsertMode = false;

		if (command.equals(Model.INSERT_COMMAND)) {
			this.tryToSetInsertMode(tokens);
		} else {
			if (command.equals(Model.DELETE_COMMAND)) {
				this.tryToDelete(tokens);
			} else if (command.equals(Model.SETLINE_COMMAND)) {
				this.tryToSetCurrentLine(tokens);
			} else if (command.equals(Model.DONE_COMMAND)) {
				this.tryToBeDone(tokens);
			} else if (command.equals(Model.DISPLAY_COMMAND)) {
				this.tryToDisplay(tokens);
			} else if (command.equals(Model.FIND_COMMAND)) {
				this.tryToFindWord(tokens);
			} else if (command.equals(Model.REPLACE_COMMAND)) {
				this.tryToReplaceWord(tokens);
			} else if (command.equals(Model.INSERT_WORD_COMMAND)) {
				this.tryToInsertWord(tokens);
			} else if (command.equals(Model.DELETE_WORD_COMMAND)) {
				this.tryToDeleteWord(tokens);
			} else if (command.equals(Model.LAST_LINE_COMMAND)) {
				this.tryToPrintLastLine(tokens);
			} else {
				this.displayError(Model.BAD_COMMAND);
			}
		}
	}

	// check for errors when processing the $DISPLAY_LINES command
	private void tryToDisplay(StringTokenizer tokens) {
		int from,to;

		// replace $LAST_LINE with last line number in document
		tokens = this.replaceLastLine(tokens);

		if(tokens.countTokens() != 0 && tokens.countTokens() != 2) {
			this.displayError(Model.NO_ARGUMENTS_EXPECTED + " OR " + Model.TWO_ARGUMENTS_EXPECTED);
		} else {
			if (tokens.countTokens() == 0) {
				this.display(0, this.document.getTotalNumberOfLines()-1);
			} else {
				try {
					from = Integer.parseInt(tokens.nextToken());
					to = Integer.parseInt(tokens.nextToken());
					if ((from < 0) || (from > to) || (to >= this.document.getTotalNumberOfLines())) {
						this.displayError(Model.OUT_OF_RANGE);
					} else {
						this.display(from, to);
					}
				} catch(NumberFormatException exception) {
					displayError(Model.INTEGER_EXPECTED);
				}
			}
		}
	}

	// display the lines from index 'from' to index 'to'
	private void display(int from, int to) {
		String string = "<html><body>\n", line;
		int loop;

		if (from > 0) {
			// add all the lines before from as non-bold
			string = string + this.prepareBuffer(0, from);
		}

		for (loop = from; loop <= to; loop++) {
			line = this.document.getLineAt(loop);

			if (loop == this.document.getCurrentLineNumber()) {
				// highlight the current line blue
				string = string + "<font color=blue><strong>" + line + "</strong></font><br>\n";
			} else {
				// highlight lines from-to as bold (black usually, but not necessarily)
				string = string + "<strong>" + line + "</strong><br>\n";
			}
		}

		if (to < document.getTotalNumberOfLines()) {
			// add all the lines after to as non-bold
			string = string + this.prepareBuffer((to+1), this.document.getTotalNumberOfLines());
		}

		string += "\n</body></html>";

		// refresh the display with the new buffer
		this.view.updateView(string);
	}

	// display all the lines in the document in standard text (i.e. non-bold variable-width)
	private void displayAllLines() {
		String string = this.prepareBuffer();

		// always refresh the display
		this.view.updateView(string);
	}

	// check for errors when executing the $INSERT_LINES command
        private void tryToSetInsertMode(StringTokenizer tokens) {
		if (tokens.countTokens() != 0) {
			this.displayError(Model.NO_ARGUMENTS_EXPECTED);
		} else {
			// refresh the display
			this.view.updateView(" ", false);
			this.inInsertMode = true;
		}
	}

	// insert the line 'string' into the document
	private void insert(String string) throws IndexOutOfBoundsException {
		this.document.insertLine(string, this.document.getCurrentLineNumber());

		// always refresh the display
		this.displayAllLines();
	}

	// check for errors when processing the $DELETE_LINES command
	private void tryToDelete(StringTokenizer tokens) {
		int from,to;

		// replace $LAST_LINE with last line number in document
		tokens = this.replaceLastLine(tokens);

		try {
			if (tokens.countTokens() != 2) {
				this.displayError(Model.TWO_INTEGERS_EXPECTED);
			} else {
				from = Integer.parseInt(tokens.nextToken());
				to = Integer.parseInt(tokens.nextToken());
				this.delete(from, to);
			}
		} catch (IndexOutOfBoundsException exception) {
			this.displayError(Model.MALFORMED_RANGE);
		} catch (NumberFormatException exception) {
			this.displayError(Model.INTEGER_FORMAT_ERROR);
		}
	}

	// delete the lines from index 'from' to index 'to'
	private void delete(int from, int to) throws IndexOutOfBoundsException {
		this.document.deleteLines(from, to);

		// always refresh the display
		this.displayAllLines();
	}

	// check for errors when processing the $SET_LINE command
	private void tryToSetCurrentLine(StringTokenizer tokens) {
		int lineNumber;

		// replace $LAST_LINE with last line number in document
		tokens = this.replaceLastLine(tokens);

		try {
			if (tokens.countTokens() != 1) {
				this.displayError(Model.ONE_INTEGER_EXPECTED);
			} else {
				lineNumber = Integer.parseInt(tokens.nextToken());
				this.setCurrentLineNumber(lineNumber);
			}
		} catch (IndexOutOfBoundsException exception) {
			this.displayError(Model.OUT_OF_RANGE);
		} catch (NumberFormatException exception) {
			this.displayError(Model.INTEGER_FORMAT_ERROR);
		}
	}

	// set the current line to 'lineNumber'
	private void setCurrentLineNumber(int lineNumber) throws IndexOutOfBoundsException {
		this.document.setCurrentLineNumber(lineNumber);

		// always refresh the display
		this.displayAllLines();
	}

	// check for errors when trying to execute the $FIND_WORD command
	private void tryToFindWord(StringTokenizer tokens) {
		if (tokens.countTokens() != 1) {
			this.displayError(Model.ONE_ARGUMENT_EXPECTED);
		} else {
			this.findWord(tokens.nextToken());
		}
	}

	// find the word 'word'
	private void findWord(String word) {
		int loop, lineCount = this.document.getTotalNumberOfLines();
		StringTokenizer tokens;
		boolean atStart;
		String buffer = "", line, currentWord, currentLine = "";

		// this sets the current line number to the first line found with the word
		this.document.find(word);

		for (loop = 0; loop < lineCount; loop++) {
			line = this.document.getLineAt(loop);
			tokens = new StringTokenizer(line);
			atStart = true;

			if (!tokens.hasMoreTokens()) {
				// take care of blank lines (don't want any exceptions thrown)
				currentLine = line;
			}

			while (tokens.hasMoreTokens()) {
				currentWord = tokens.nextToken();

				if (currentWord.equals(word)) {
					// found words are highlighted green
					currentWord = "<font color=green><strong>" + currentWord + "</strong></font>";
				}

				if (atStart) {
					currentLine = currentWord;
					atStart = false;
				} else {
					currentLine = currentLine + " " + currentWord;
				}
			}

			if (loop == this.document.getCurrentLineNumber()) {
				// The current line is highlighted blue
				currentLine = "<font color=blue><strong>" + currentLine + "</strong></font>";
			}

			buffer = buffer + currentLine + "<br>\n";
		}

		// refresh the display
		this.view.updateView(buffer);
	}

	// check for errors while processing the $LAST_LINE command
	private void tryToPrintLastLine(StringTokenizer tokens) {
		if(tokens.countTokens() != 0) {
			this.displayError(Model.NO_ARGUMENTS_EXPECTED);
		} else if (document.getTotalNumberOfLines() == 0) {
			this.displayError(Model.NO_LINES_PRESENT);
		} else {
			this.printLastLine();
		}
	}

	// display the line number of the last line in the list
        private void printLastLine() {
		// always refresh the display
		this.displayAllLines();

		this.view.updateView("The Last Line is " + (this.document.getTotalNumberOfLines()-1), false);
	}

	// check for errors while processing the $INSERT_WORD command
	private void tryToInsertWord(StringTokenizer tokens) {
		String string, indexString;
		int index, currentLineNumber;

		if (tokens.countTokens() != 2) {
			this.displayError(Model.TWO_ARGUMENTS_EXPECTED);
		} else {
			try {
				string = tokens.nextToken();
				indexString = tokens.nextToken();
				index = Integer.parseInt(indexString);
				currentLineNumber = this.document.getCurrentLineNumber();

				this.insertWord(index, string, document.getCurrentLineNumber());
			} catch (IndexOutOfBoundsException exception) {
				this.displayError(Model.OUT_OF_RANGE);
			} catch (NumberFormatException exception) {
				this.displayError(Model.INTEGER_FORMAT_ERROR);
			}
		}
	}

	// insert word at index 'index' into line 'lineNumber'
        private void insertWord(int index, String string, int lineNumber) throws IndexOutOfBoundsException {
		this.document.insertWordInLine(index, string, lineNumber);

		// always refresh the display
		this.displayAllLines();
	}

	// check for errors while executing the $DELETE_WORD command
	private void tryToDeleteWord(StringTokenizer tokens) {
		String string;

		if (tokens.countTokens() != 1) {
			this.displayError(Model.ONE_ARGUMENT_EXPECTED);
		} else {
			try {
				string = tokens.nextToken();
				this.deleteWord(string, this.document.getCurrentLineNumber());
			} catch(IndexOutOfBoundsException exception) {
				this.displayError(Model.OUT_OF_RANGE);
			}
		}
	}

	// delete the word 'string' from the line 'lineNumber'
	private void deleteWord(String string, int lineNumber) {
		this.document.deleteWordFromLine(string, lineNumber);

		// always refresh the display
		this.displayAllLines();
	}

	// check for errors while processing the $REPLACE_WORD command
	private void tryToReplaceWord(StringTokenizer tokens) {
		String replacee, replacer;

		if (tokens.countTokens() != 2) {
			this.displayError(Model.TWO_ARGUMENTS_EXPECTED);
		} else {
			replacee = tokens.nextToken();
			replacer = tokens.nextToken();
			this.replaceWord(replacee, replacer);
		}
	}

	// replace the word 'replacee' with the word 'replacer'
        private void replaceWord(String replacee, String replacer) {
		this.document.replaceWord(replacee, replacer);

		// refresh the display
		this.displayAllLines();
	}

	// check for errors while processing the $DONE command
	private void tryToBeDone(StringTokenizer tokens) {
		if (tokens.countTokens() != 0) {
			this.displayError(Model.NO_ARGUMENTS_EXPECTED);
		} else {
			this.done();
		}
	}

	// quit the LineEditor
	private void done() {
		System.exit(0);
	}
}
