/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package model;

import datatypes.Dictionary;
import datastructures.BinarySearchTree;

//////////////////////////////////////////////////////////////////////
/**
 * A WordDictionary is a dictionary that only allows for operations
 * on Words.  Words may be inserted, deleted and found.
 */
public class WordDictionary {
    private Dictionary wordDictionary;

    //////////////////////////////////////////////////////////////////////
    /**
     * Create a WordDictionary wrapping the supplied dictionary.
     *
     * @param the dictionary used to store the words
     **/
    public WordDictionary(Dictionary dictionary)
        {
        this.wordDictionary = dictionary;
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Create a new WordDictionary object.
     **/
    public WordDictionary()
        {
        this.wordDictionary = new BinarySearchTree();
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Insert the specified word into the dictionary.
     * 
     * @param currentWord the word to insert
     **/
    public void insertWord(Word currentWord)
        {
	WordList currentWordList;
	Object treeObject = wordDictionary.find(currentWord.getValue());
        
	//If the word's value is not in the tree then it returns null.
	//We need to put it in there. 
	if(treeObject == null)
            {
	    currentWordList = new WordList(currentWord.getValue());
	    wordDictionary.insert(currentWord.getValue(), currentWordList);
            }
	else
            {
	    currentWordList = (WordList)treeObject;
            }
	currentWordList.insert(currentWord);
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Returns a list of all words matching the specified value.
     * If no words are found, an empty WordList is returned.
     * 
     * @param value the word to search for
     * @return an array of words matching value
     **/
    public Word[] findAllWords(String value)
        {
	Object treeObject = wordDictionary.find(value);
	WordList wordList;
	if(treeObject == null)
            {
	    wordList = new WordList(value);
            }
	else
            {
	    wordList = (WordList)treeObject;
            }
        Word[] words = new Word[wordList.size()];
        for (int i = 0; i < words.length; i++)
            {
            words[i] = wordList.getWordAt(i);
            }
        return words;        
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Delete the specified word from the dictionary.
     * If the word is not in the dictionary, nothing happens.
     * 
     * @param currentWord the word to delete.
     */
    public void deleteWord(Word currentWord)
        {
	WordList currentWordList = 
	    (WordList)wordDictionary.find(currentWord.getValue());
	//delete the word from the list.
	currentWordList.delete(currentWord);
	
	//If the word list is empty then delete it from the tree.
	if(currentWordList.size() == 0)
            {
            wordDictionary.delete(currentWord.getValue());
            }  
        }
}
