/*
 * Copyright (C) 2000 John David Ratliff, Jason Andrew Fisk, and Hongwei Guan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package model;

import datastructures.DoublyLinkedList;

//////////////////////////////////////////////////////////////////////
/**
 * This class takes a string on initialization that serves as a basis for 
 * future comparisions.  A WordList object will only allow insertions of 
 * Word objects that have the same value as this string.  If any other 
 * Word objects are attempted to be inserted, then an exception is thrown.
 */
//////////////////////////////////////////////////////////////////////
public class WordList
    {
    //This class delegates all of its list like behavior to a
    //DoublyLinkedList object.
    private DoublyLinkedList list;

    //The String to be used for comparisons.
    private String value;

    //////////////////////////////////////////////////////////////////////
    /**
     * The string that is passed in to the constructor allows us to make
     * sure that allow Word objects inserted into it WordList have the same
     * value as this String.
     */
    public WordList(String value)
        {
	this.value = value;
	
	list = new DoublyLinkedList();
        }

    //////////////////////////////////////////////////////////////////////
    /** 
     * Insert the given Word if the Word's value is the same as our value.
     * If inserted, then it is inserted at the beginning of the list.
     * If the Word's value is not the same then this method throws exceptions
     *
     *@exception java.lang.IllegalArgumentException Thrown when the Word's 
     *      getValue() is not equal to the value of this WordList.
     */
     public void insert(Word word) throws IllegalArgumentException
        {
	if(!word.getValue().equals(this.getValue()))
            {
	    throw new IllegalArgumentException(word.getValue() + " != " +
					       this.getValue());
            }
	else
            {
	    list.insert(0, word);
            }
        }

    //////////////////////////////////////////////////////////////////////
    /** 
     * Delete the given Word if the Word's value is the same as our value.
     * If the Word's value is not the same then this method throws an exception
     *
     *@exception java.lang.IllegalArgumentException Thrown when the Word's 
     *      getValue() is not equal to the value of this WordList.
     */
    public void delete(Word word) throws IllegalArgumentException
        {
	if(!word.getValue().equals(this.getValue()))
            {
	    throw new IllegalArgumentException(word.getValue() + " != " +
					       this.getValue());
            }
	else
            {
	    int index = indexOf(word);
	    list.delete(index);
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the value of this word list. Only Words with the same value 
     * can be inserted in this WordList.
     */
    public String getValue()
        {
	return value;
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the size of this list.
     */
    public int size()
        {
	return list.size();
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Get a Word at the specified index.
     *
     *@exception java.lang.IndexOutOfBoundsException Thrown when the index is
     * either larger than the size of this list, or is negative.
     */
    public Word getWordAt(int index) throws IndexOutOfBoundsException
        {
	if(index < 0 || index >= size())
            {
	    throw new IndexOutOfBoundsException(index +" is out of range");
	    //return null;//nothing to return
            }
	else
            {
	    return (Word)list.get(index);
            }
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Returns true if this WordList contains the given Word.
     */
    public boolean contains(Word word)
        {
	return (list.indexOf(word) != -1);
        }

    //////////////////////////////////////////////////////////////////////
    /**
     * Return the index of the given word.  Returns -1 if it is not in this
     * WordList.
     */
    public int indexOf(Word word)
        {
	return list.indexOf(word);
        }
    public String toString()
        {
        return getValue();
        }
    }
